@@ -2,7 +2,7 @@
  *
  *  AT chat library with GLib integration
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -40,7 +40,6 @@
 static const char *none_prefix[] = { NULL };
 
 static void g_at_chat_wakeup_writer(GAtChat *chat);
-static void debug_chat(GAtChat *chat, gboolean in, const char *str, gsize len);
 
 struct at_command {
 	char *cmd;
@@ -71,6 +70,7 @@ struct _GAtChat {
 	guint next_notify_id;			/* Next notify id */
 	guint read_watch;			/* GSource read id, 0 if none */
 	guint write_watch;			/* GSource write id, 0 if none */
+	gboolean use_write_watch;		/* watch usage for non blocking */
 	GIOChannel *channel;			/* channel */
 	GQueue *command_queue;			/* Command queue */
 	guint cmd_bytes_written;		/* bytes written from cmd */
@@ -79,6 +79,7 @@ struct _GAtChat {
 	gpointer user_disconnect_data;		/* user disconnect data */
 	struct ring_buffer *buf;		/* Current read buffer */
 	guint read_so_far;			/* Number of bytes processed */
+	guint max_read_attempts;		/* max number of read attempts */
 	GAtDebugFunc debugf;			/* debugging output function */
 	gpointer debug_data;			/* Data to pass to debug func */
 	char *pdu_notify;			/* Unsolicited Resp w/ PDU */
@@ -311,7 +312,6 @@ static gboolean g_at_chat_match_notify(GAtChat *chat, char *line)
 {
 	GHashTableIter iter;
 	struct at_notify *notify;
-	char *prefix;
 	gpointer key, value;
 	gboolean ret = FALSE;
 	GAtResult result;
@@ -321,7 +321,6 @@ static gboolean g_at_chat_match_notify(GAtChat *chat, char *line)
 	result.final_or_pdu = 0;
 
 	while (g_hash_table_iter_next(&iter, &key, &value)) {
-		prefix = key;
 		notify = value;
 
 		if (!g_str_has_prefix(line, key))
@@ -352,8 +351,7 @@ static gboolean g_at_chat_match_notify(GAtChat *chat, char *line)
 	return ret;
 }
 
-static void g_at_chat_finish_command(GAtChat *p, gboolean ok,
-						char *final)
+static void g_at_chat_finish_command(GAtChat *p, gboolean ok, char *final)
 {
 	struct at_command *cmd = g_queue_pop_head(p->command_queue);
 	GSList *response_lines;
@@ -555,7 +553,7 @@ static void have_pdu(GAtChat *p, char *pdu)
 	gboolean listing_pdu = FALSE;
 
 	if (!pdu)
-		goto err;
+		goto error;
 
 	result.lines = g_slist_prepend(NULL, p->pdu_notify);
 	result.final_or_pdu = pdu;
@@ -580,7 +578,7 @@ static void have_pdu(GAtChat *p, char *pdu)
 
 	g_slist_free(result.lines);
 
-err:
+error:
 	g_free(p->pdu_notify);
 	p->pdu_notify = NULL;
 
@@ -686,81 +684,6 @@ static void new_bytes(GAtChat *p)
 	g_at_chat_unref(p);
 }
 
-static void debug_chat(GAtChat *chat, gboolean in, const char *str, gsize len)
-{
-	char type = in ? '<' : '>';
-	gsize escaped = 2; /* Enough for '<', ' ' */
-	char *escaped_str;
-	const char *esc = "<ESC>";
-	gsize esc_size = strlen(esc);
-	const char *ctrlz = "<CtrlZ>";
-	gsize ctrlz_size = strlen(ctrlz);
-	gsize i;
-
-	if (!chat->debugf || !len)
-		return;
-
-	for (i = 0; i < len; i++) {
-		char c = str[i];
-
-		if (isprint(c))
-			escaped += 1;
-		else if (c == '\r' || c == '\t' || c == '\n')
-			escaped += 2;
-		else if (c == 26)
-			escaped += ctrlz_size;
-		else if (c == 25)
-			escaped += esc_size;
-		else
-			escaped += 4;
-	}
-
-	escaped_str = g_malloc(escaped + 1);
-	escaped_str[0] = type;
-	escaped_str[1] = ' ';
-	escaped_str[2] = '\0';
-	escaped_str[escaped] = '\0';
-
-	for (escaped = 2, i = 0; i < len; i++) {
-		char c = str[i];
-
-		switch (c) {
-		case '\r':
-			escaped_str[escaped++] = '\\';
-			escaped_str[escaped++] = 'r';
-			break;
-		case '\t':
-			escaped_str[escaped++] = '\\';
-			escaped_str[escaped++] = 't';
-			break;
-		case '\n':
-			escaped_str[escaped++] = '\\';
-			escaped_str[escaped++] = 'n';
-			break;
-		case 26:
-			strncpy(&escaped_str[escaped], ctrlz, ctrlz_size);
-			escaped += ctrlz_size;
-			break;
-		case 25:
-			strncpy(&escaped_str[escaped], esc, esc_size);
-			escaped += esc_size;
-			break;
-		default:
-			if (isprint(c))
-				escaped_str[escaped++] = c;
-			else {
-				escaped_str[escaped++] = '\\';
-				escaped_str[escaped++] = '0' + ((c >> 6) & 07);
-				escaped_str[escaped++] = '0' + ((c >> 3) & 07);
-				escaped_str[escaped++] = '0' + (c & 07);
-			}
-		}
-	}
-
-	chat->debugf(escaped_str, chat->debug_data);
-	g_free(escaped_str);
-}
-
 static gboolean received_data(GIOChannel *channel, GIOCondition cond,
 				gpointer data)
 {
@@ -770,30 +693,34 @@ static gboolean received_data(GIOChannel *channel, GIOCondition cond,
 	gsize rbytes;
 	gsize toread;
 	gsize total_read = 0;
+	guint read_count = 0;
 
 	if (cond & G_IO_NVAL)
 		return FALSE;
 
 	/* Regardless of condition, try to read all the data available */
 	do {
-		rbytes = 0;
-
 		toread = ring_buffer_avail_no_wrap(chat->buf);
 
 		if (toread == 0)
 			break;
 
+		rbytes = 0;
 		buf = ring_buffer_write_ptr(chat->buf);
 
 		err = g_io_channel_read(channel, (char *) buf, toread, &rbytes);
-		debug_chat(chat, TRUE, (char *)buf, rbytes);
+		g_at_util_debug_chat(TRUE, (char *)buf, rbytes,
+					chat->debugf, chat->debug_data);
+
+		read_count++;
 
 		total_read += rbytes;
 
 		if (rbytes > 0)
 			ring_buffer_write_advance(chat->buf, rbytes);
 
-	} while (err == G_IO_ERROR_NONE && rbytes > 0);
+	} while (err == G_IO_ERROR_NONE && rbytes > 0 &&
+					read_count < chat->max_read_attempts);
 
 	if (total_read > 0)
 		new_bytes(chat);
@@ -801,7 +728,7 @@ static gboolean received_data(GIOChannel *channel, GIOCondition cond,
 	if (cond & (G_IO_HUP | G_IO_ERR))
 		return FALSE;
 
-	if (err != G_IO_ERROR_NONE && err != G_IO_ERROR_AGAIN)
+	if (read_count > 0 && rbytes == 0 && err != G_IO_ERROR_AGAIN)
 		return FALSE;
 
 	return TRUE;
@@ -829,6 +756,9 @@ static gboolean wakeup_no_response(gpointer user)
 	if (chat->debugf)
 		chat->debugf("Wakeup got no response\n", chat->debug_data);
 
+	if (cmd == NULL)
+		return FALSE;
+
 	g_at_chat_finish_command(chat, FALSE, NULL);
 	cmd = at_command_create(chat->wakeup, none_prefix, FALSE,
 				NULL, wakeup_cb, chat, NULL, TRUE);
@@ -932,8 +862,8 @@ static gboolean can_write_data(GIOChannel *channel, GIOCondition cond,
 		return FALSE;
 	}
 
-	debug_chat(chat, FALSE, cmd->cmd + chat->cmd_bytes_written,
-			bytes_written);
+	g_at_util_debug_chat(FALSE, cmd->cmd + chat->cmd_bytes_written,
+				bytes_written, chat->debugf, chat->debug_data);
 	chat->cmd_bytes_written += bytes_written;
 
 	if (bytes_written < towrite)
@@ -951,17 +881,22 @@ static void g_at_chat_wakeup_writer(GAtChat *chat)
 	if (chat->write_watch != 0)
 		return;
 
-	chat->write_watch = g_io_add_watch_full(chat->channel,
+	if (chat->use_write_watch == TRUE) {
+		chat->write_watch = g_io_add_watch_full(chat->channel,
 				G_PRIORITY_DEFAULT,
 				G_IO_OUT | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
 				can_write_data, chat,
 				(GDestroyNotify)write_watcher_destroy_notify);
+	} else {
+		while (can_write_data(chat->channel, G_IO_OUT, chat) == TRUE);
+		write_watcher_destroy_notify(chat);
+	}
 }
 
-GAtChat *g_at_chat_new(GIOChannel *channel, GAtSyntax *syntax)
+static GAtChat *create_chat(GIOChannel *channel, GIOFlags flags,
+				GAtSyntax *syntax)
 {
 	GAtChat *chat;
-	GIOFlags io_flags;
 
 	if (!channel)
 		return NULL;
@@ -979,6 +914,14 @@ GAtChat *g_at_chat_new(GIOChannel *channel, GAtSyntax *syntax)
 	chat->next_notify_id = 1;
 	chat->debugf = NULL;
 
+	if (flags & G_IO_FLAG_NONBLOCK) {
+		chat->use_write_watch = TRUE;
+		chat->max_read_attempts = 3;
+	} else {
+		chat->use_write_watch = FALSE;
+		chat->max_read_attempts = 1;
+	}
+
 	chat->buf = ring_buffer_new(4096);
 
 	if (!chat->buf)
@@ -992,20 +935,9 @@ GAtChat *g_at_chat_new(GIOChannel *channel, GAtSyntax *syntax)
 	chat->notify_list = g_hash_table_new_full(g_str_hash, g_str_equal,
 				g_free, (GDestroyNotify)at_notify_destroy);
 
-	if (g_io_channel_set_encoding(channel, NULL, NULL) !=
-			G_IO_STATUS_NORMAL)
+	if (!g_at_util_setup_io(channel, flags))
 		goto error;
 
-	io_flags = g_io_channel_get_flags(channel);
-
-	io_flags |= G_IO_FLAG_NONBLOCK;
-
-	if (g_io_channel_set_flags(channel, io_flags, NULL) !=
-			G_IO_STATUS_NORMAL)
-		goto error;
-
-	g_io_channel_set_close_on_unref(channel, TRUE);
-
 	chat->channel = channel;
 	chat->read_watch = g_io_add_watch_full(channel, G_PRIORITY_DEFAULT,
 				G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
@@ -1030,6 +962,16 @@ error:
 	return NULL;
 }
 
+GAtChat *g_at_chat_new(GIOChannel *channel, GAtSyntax *syntax)
+{
+	return create_chat(channel, G_IO_FLAG_NONBLOCK, syntax);
+}
+
+GAtChat *g_at_chat_new_blocking(GIOChannel *channel, GAtSyntax *syntax)
+{
+	return create_chat(channel, 0, syntax);
+}
+
 GIOChannel *g_at_chat_get_channel(GAtChat *chat)
 {
 	if (chat == NULL)
@@ -1315,7 +1257,6 @@ gboolean g_at_chat_unregister(GAtChat *chat, guint id)
 {
 	GHashTableIter iter;
 	struct at_notify *notify;
-	char *prefix;
 	gpointer key, value;
 	GSList *l;
 
@@ -1325,7 +1266,6 @@ gboolean g_at_chat_unregister(GAtChat *chat, guint id)
 	g_hash_table_iter_init(&iter, chat->notify_list);
 
 	while (g_hash_table_iter_next(&iter, &key, &value)) {
-		prefix = key;
 		notify = value;
 
 		l = g_slist_find_custom(notify->nodes, GUINT_TO_POINTER(id),
@@ -1350,7 +1290,6 @@ gboolean g_at_chat_unregister_all(GAtChat *chat)
 {
 	GHashTableIter iter;
 	struct at_notify *notify;
-	char *prefix;
 	gpointer key, value;
 	GSList *l;
 
@@ -1360,14 +1299,13 @@ gboolean g_at_chat_unregister_all(GAtChat *chat)
 	g_hash_table_iter_init(&iter, chat->notify_list);
 
 	while (g_hash_table_iter_next(&iter, &key, &value)) {
-		prefix = key;
 		notify = value;
 
 		for (l = notify->nodes; l; l = l->next)
 			at_notify_node_destroy(l->data);
 
 		g_slist_free(notify->nodes);
-		notify->nodes=  NULL;
+		notify->nodes = NULL;
 		g_hash_table_iter_remove(&iter);
 	}
 
